#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU    8000000       
#include <util/delay.h>
#include <avr/sleep.h>

#define LEDUNSIX       1     // Leds 1 et 6 sur PB1
#define LEDTROISQUATRE 0     // Leds 3 et 4 sur PB5
#define LEDDEUXCINQ    4     // Leds 2 et 5 sur PB4
#define LEDSEPT        3     // Leds 7 sur PB3
#define UNESECONDE     1000  // 1 seconde égal à 1000ms

volatile uint8_t roll=0, initSeed=0, seed=0;

ISR(INT0_vect){
   MCUCR &= ~(1<<SE); // Idle of sleep mode -> disable
   roll=1;
   if(initSeed==0){
    randomSeed(seed);
    initSeed=1;
   }
   
}

void action(){
  
  uint8_t tirage, newTirage;
  for (uint8_t i=0;i<random(10,21);i++){
      do{
        newTirage = random(1,7);
      }while(tirage == newTirage);
      tirage = newTirage;  
    
    onEteind();
    afficheTirage(tirage);        // On affiche le tirage
    _delay_ms(UNESECONDE/5);                 
  }

  // Phase de tirage réel
  onEteind();
  do{
    newTirage = random(1,7);
  }while(tirage == newTirage); 
  
  afficheTirage(newTirage);
  _delay_ms(UNESECONDE*10);             // Tirage final, on fait une pause de 7 secondes.*/
  onEteind();
  roll=0;
}

void init_ISR(){
    PORTB |= 1<<PB2;//|1<<PB5;   // PULL UP (PB2) AND USUED PIN PB5
    MCUCR |= 1<<ISC01; // Falling EDGE INT0 (PB2)
    GIMSK |= 1<<INT0;  // Activate the INT0
    sei();             //Active General Interrupt
}

void init_GPIO(){
  // On configure les PINs en sortie pour les LEDs
  DDRB |= (1<<LEDUNSIX)|(1<<LEDTROISQUATRE)|(1<<LEDDEUXCINQ)|(1<<LEDSEPT);
}

void onEteind(){
  PORTB &= ~((1<<LEDUNSIX)|(1<<LEDTROISQUATRE)|(1<<LEDDEUXCINQ)|(1<<LEDSEPT));
  _delay_ms(UNESECONDE/5);
}

void afficheTirage(uint8_t tirage){
    PORTB &= ~((1<<LEDUNSIX)|(1<<LEDTROISQUATRE)|(1<<LEDDEUXCINQ)|(1<<LEDSEPT));
    switch (tirage) {
      case 1:
        PORTB |= (1<<LEDSEPT);      
        break;
      case 2:
        PORTB |= (1<<LEDUNSIX);      
        break;
      case 3:
        PORTB |= (1<<LEDUNSIX)|(1<<LEDSEPT);
        break;
      case 4:
        PORTB |= (1<<LEDUNSIX)|(1<<LEDDEUXCINQ);   
        break;
      case 5:
        PORTB |= (1<<LEDUNSIX)|(1<<LEDDEUXCINQ)|(1<<LEDSEPT);      
        break;
      case 6:
        PORTB |= (1<<LEDUNSIX)|(1<<LEDDEUXCINQ)|(1<<LEDTROISQUATRE);     
        break;
        
      default:
        // if nothing else matches, do the default
        // default is optional
        break;
    }
}

int main() {
  init_GPIO();
  init_ISR();
  
  while(1){
    if (initSeed==0) seed++;
    
    if (roll==1){
      action();
      MCUCR |= (1<<SE); // Idle Mode of sleep mode
      sleep_mode();    
    }
  }

  return 0;
}
